@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2024, NVIDIA CORPORATION.
+ * Copyright (c) 2013-2023, NVIDIA CORPORATION.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -34,13 +34,11 @@
 #define NV_CTL_DEVICE_NUM                    255
 #define NV_MODESET_MINOR_DEVICE_NUM          254
 #define NV_NVSWITCH_CTL_MINOR                255
-#define NV_VGPU_VFIO_CTL_MINOR               0
 
 #define NV_DEVICE_FILE_PATH "/dev/nvidia%d"
 #define NV_CTRL_DEVICE_FILE_PATH "/dev/nvidiactl"
 #define NV_MODESET_DEVICE_NAME "/dev/nvidia-modeset"
 #define NV_VGPU_VFIO_DEVICE_NAME "/dev/nvidia-vgpu%d"
-#define NV_VGPU_VFIO_CTL_NAME "/dev/nvidia-vgpuctl"
 #define NV_NVLINK_DEVICE_NAME "/dev/nvidia-nvlink"
 #define NV_NVSWITCH_CTL_NAME "/dev/nvidia-nvswitchctl"
 #define NV_NVSWITCH_DEVICE_NAME "/dev/nvidia-nvswitch%d"
@@ -48,6 +46,10 @@
 #define NV_CAPS_MODULE_NAME "nvidia-caps"
 #define NV_CAP_DEVICE_NAME "/dev/" NV_CAPS_MODULE_NAME "/nvidia-cap%d"
 
+#define NV_CAPS_IMEX_CHANNELS_MODULE_NAME "nvidia-caps-imex-channels"
+#define NV_CAPS_IMEX_CHANNEL_DEVICE_NAME \
+        "/dev/" NV_CAPS_IMEX_CHANNELS_MODULE_NAME "/channel%d"
+
 #define NV_CHAR_DEVICE_NAME "/dev/char/%d:%d"
 
 #if defined(NV_LINUX)
@@ -85,6 +87,8 @@ int nvidia_nvswitch_mknod(int minor);
 int nvidia_nvswitch_get_file_state(int minor);
 int nvidia_cap_mknod(const char* cap_file_path, int *minor);
 int nvidia_cap_get_file_state(const char* cap_file_path);
+int nvidia_cap_imex_channel_mknod(int minor);
+int nvidia_cap_imex_channel_file_state(int minor);
 int nvidia_get_chardev_major(const char *name);
 int nvidia_msr_modprobe(void);
 int nvidia_enable_auto_online_movable(const int print_errors);
